package com.tejosma.pform.common.jms;

import javax.jms.ExceptionListener;
import javax.jms.JMSException;
import javax.jms.MessageListener;
import javax.jms.Queue;
import javax.jms.QueueBrowser;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueReceiver;
import javax.jms.QueueSession;
import javax.naming.InitialContext;

import org.apache.log4j.Category;

import com.tejosma.pform.common.config.AppConfig;

public abstract class JMSReceiver implements MessageListener,ExceptionListener {

	private QueueConnection conn;

	private QueueSession session;

	private Queue que;
	
	private QueueReceiver recv;

	private String queueName = "";

	private Category logger = Category.getInstance(JMSReceiver.class.getName());

	private static InitialContext iniCtx = null;
	private static QueueConnectionFactory qcf = null;
	
	// service locator patter implemented
	static{
		try {
			iniCtx = new InitialContext();
			qcf = (QueueConnectionFactory) iniCtx.lookup(AppConfig.getCONNECTION_FACTORY());
		} catch (Exception e) {
			Category logger = Category.getInstance(JMSSender.class.getName());
			logger.error("static block connection issue");
		}
	}
	
	public void start(String qName) {
		queueName = JMSUtil.jmsQNamePrefix + qName;
		setupPoint2Point();
		initReciever();
	}

	// public abstract void callInit();
	private boolean setupPoint2Point() {
		try {
			conn = qcf.createQueueConnection();
			conn.setExceptionListener(this);
			que = (Queue) iniCtx.lookup(queueName);
			session = conn.createQueueSession(false, QueueSession.AUTO_ACKNOWLEDGE);
			conn.start();
			return true;
		} catch (Exception e) {
			logger.error(e.getMessage());
			return false;
		}
	}

	public String getQueueName() {
		return queueName;
	}

	public void initReciever() {
		try {
			recv = session.createReceiver(que);
			recv.setMessageListener(this);
		} catch (Exception e) {
			logger.error(e.getMessage());
		}
	}

	/**
	 * This will reset the queue ie stop's and rebinds the queue
	 */
	public void resetQueue() throws Exception {
		stop();
		setupPoint2Point();
		initReciever();
	}

	/**
	 * stops and clears the connection and current session
	 * 
	 * @throws Exception
	 */
	public void stop() throws Exception {
		if (conn != null) {
			conn.stop();
			if (session != null) {
				session.close();
			}
			if(recv!=null)
			{
				recv.close();
			}
			conn.close();
		}
		recv = null;
		que = null;
		session = null;
		conn = null;
		System.out.println("Queue closed");
	}

	public QueueConnection getQueueConnection() {
		if (conn == null) {
			setupPoint2Point();
		}
		return conn;
	}

	public QueueSession getQueueSession() {
		if (session == null) {
			setupPoint2Point();
		}
		return session;
	}

	public Queue getQueue() {
		if (que == null) {
			setupPoint2Point();
		}
		return que;
	}

	/**
	 * object to look at messages on a queue without removing them
	 * 
	 * @return queueBrowser
	 * @throws JMSException
	 */
	public QueueBrowser getQueueBrowser() throws JMSException {
		QueueBrowser browser = session.createBrowser(que);
		return browser;
	}
	
	/**
	 * manage and recreate connection if broken
	 */
	public void onException(JMSException arg0)
	{
		 logger.debug("JMSReceiver:onException:"+arg0.toString());
		 try 
         {
            conn.close();  // unregisters the ExceptionListener
         }
         catch(Exception e2) {
        	 logger.error("JMSReceiver:onException:"+e2.toString());
            // I will get an Exception anyway, since the connection to the server is
            // broken, but close() frees up resources associated with the connection
         }finally{
        	 conn = null;
         }
         if(setupPoint2Point())
        	 logger.debug("JMSReceiver:onException:Can't create connection again");
	}
}